@@ -126,28 +126,8 @@ sig_fit <- function(catalogue_matrix,
     send_success("Signature index detected.")
     send_info("Checking signature database in package.")
 
-    db_file <- switch(
-      sig_db,
-      legacy = system.file("extdata", "legacy_signatures.RDs",
-        package = "maftools", mustWork = TRUE
-      ),
-      SBS = system.file("extdata", "SBS_signatures.RDs",
-        package = "maftools", mustWork = TRUE
-      ),
-      DBS = system.file("extdata", "DBS_signatures.rds",
-        package = "sigminer", mustWork = TRUE
-      ),
-      ID = system.file("extdata", "ID_signatures.rds",
-        package = "sigminer", mustWork = TRUE
-      ),
-      TSB = system.file("extdata", "TSB_signatures.rds",
-        package = "sigminer", mustWork = TRUE
-      ),
-      send_stop("Invalid parameter passing to {.code sig_db}.")
-    )
-    sigs_db <- readRDS(file = db_file)
+    sigs_db <- get_sig_db(sig_db)
     sigs <- sigs_db$db
-    sigs <- apply(sigs, 2, function(x) x / sum(x))
 
     ## Some extra processing
     if (sig_db == "legacy" & db_type == "human-genome") {
@@ -422,7 +402,6 @@ decompose_QP <- function(x, y, P, type = "absolute", ...) {
 
 
 decompose_SA <- function(x, y, P, type = "absolute", ...) {
-
   if (sum(x) != 0) {
     control <- list(...)
 
