@@ -3,7 +3,7 @@
 #' Tally a variation object like [MAF], [CopyNumber] and return a matrix for NMF de-composition and more.
 #' This is a generic function,
 #' so it can be further extended to other mutation cases.
-#' Please read details about how to set sex for identifying copy number signatures.
+#' **Please read details about how to set sex for identifying copy number signatures**.
 #' Please read <https://osf.io/s93d5/> for the generation of SBS, DBS and ID (INDEL)
 #' components. **Of note, many options are designed for method "M" only, and they are highlighted
 #' by bold fonts** (you can ignore them if you don't use "M" method).
@@ -80,8 +80,11 @@
 #' cn_tally_M <- sig_tally(cn, method = "M")
 #' # Use method designed by Wang, Shixiang et al.
 #' cn_tally_W <- sig_tally(cn, method = "W")
+#' # Use method designed by Tao & Wang.
+#' cn_tally_T <- sig_tally(cn, method = "T")
 #'
 #' expect_equal(length(cn_tally_M), length(cn_tally_W))
+#' expect_equal(length(cn_tally_T), 5L)
 #'
 #' ## for SBS
 #'
@@ -121,8 +124,9 @@ sig_tally <- function(object, ...) {
 }
 
 #' @describeIn sig_tally Returns copy number features, components and component-by-sample matrix
+#' @param indices integer vector indicating segments to keep.
 #' @param method method for feature classfication, can be one of "Macintyre" ("M"),
-#' "Wang" ("W") and "Tao & Wang" ("T").
+#' "Wang" ("W").
 #' @param feature_setting a `data.frame` used for classification.
 #' **Only used when method is "Wang" ("W")**.
 #' Default is [CN.features]. Users can also set custom input with "feature",
@@ -172,6 +176,7 @@ sig_tally <- function(object, ...) {
 sig_tally.CopyNumber <- function(object,
                                  method = "Wang",
                                  ignore_chrs = NULL,
+                                 indices = NULL,
                                  feature_setting = sigminer::CN.features,
                                  type = c("probability", "count"),
                                  reference_components = FALSE,
@@ -187,6 +192,7 @@ sig_tally.CopyNumber <- function(object,
   method <- match.arg(method, choices = c("Macintyre", "M", "Wang", "W", "Tao & Wang", "T"))
 
   if (startsWith(method, "T")) {
+    send_warning("Currently, the method 'T' is in experimental stage, please don't use it for now!")
     ## Add segment index for method "T" so the segments can be easily joined or checked
     cn_list <- get_cnlist(object, ignore_chrs = ignore_chrs, add_index = TRUE)
   } else {
@@ -287,56 +293,21 @@ sig_tally.CopyNumber <- function(object,
     feature_setting$n_obs <- colSums(cn_matrix, na.rm = TRUE)
   } else {
     # Method: Tao & Wang, 'T'
-
-    ## TODO: should also set a feature_setting dataset??
-    ## Is genome_build necessary here??
     send_info("Step: getting copy number features.")
-    cn_features <- get_features_mutex(
-      CN_data = cn_list, cores = cores,
-      genome_build = object@genome_build,
-      feature_setting = feature_setting
-    )
+    cn_features <- get_features_mutex(CN_data = cn_list, cores = cores)
     send_success("Gotten.")
 
-    ## Curretly return features and explore how to combine them
-    return(cn_features)
-    # Make order as unique(feature_setting)$feature
-    # cn_features <- cn_features[unique(feature_setting$feature)]
+    send_info("Step: generating copy number components based on combination.")
+    cn_components <- get_components_mutex(cn_features)
+    send_success("Classified and combined.")
 
-    # send_info("Step: generating copy number components based on combination.")
-    # # Check feature setting
-    # if (!inherits(feature_setting, "sigminer.features")) {
-    #   feature_setting <- get_feature_components(feature_setting)
-    # }
-    # send_success("{.code feature_setting} checked.")
-    #
-    # send_info("Step: counting components.")
-    # cn_components <- purrr::map2(cn_features, names(cn_features),
-    #                              count_components_wrapper,
-    #                              feature_setting = feature_setting
-    # )
-    # send_success("Counted.")
-    #
-    # ## Remove BoChr value is 0 in features
-    # if ("BoChr" %in% names(cn_features)) {
-    #   cn_features$BoChr <- cn_features$BoChr[cn_features$BoChr$value != 0]
-    # }
-    #
-    # send_info("Step: generating components by sample matrix.")
-    # cn_matrix <- data.table::rbindlist(cn_components, fill = TRUE, use.names = TRUE) %>%
-    #   dplyr::as_tibble() %>%
-    #   tibble::column_to_rownames(var = "component") %>%
-    #   as.matrix()
-    # # Order the matrix as feature_setting
-    # cn_matrix <- cn_matrix[feature_setting$component, ] %>%
-    #   t()
-    #
-    # if (any(is.na(cn_matrix))) {
-    #   send_warning("{.code NA} detected. There may be an issue, please contact the developer!")
-    #   send_warning("Data will still returned, but please take case of it.")
-    # }
-    # # cn_matrix[is.na(cn_matrix)] <- 0L
-    # feature_setting$n_obs <- colSums(cn_matrix, na.rm = TRUE)
+    send_info("Step: generating components by sample matrix.")
+    cn_matrix_list <- get_matrix_mutex(cn_components, indices = indices)
+    cn_matrix <- cn_matrix_list$s_mat
+
+    if (keep_only_matrix) {
+      send_info("When keep_only_matrix is TRUE, only standard matrix kept.")
+    }
   }
 
   send_success("Matrix generated.")
@@ -349,23 +320,41 @@ sig_tally.CopyNumber <- function(object,
         cn_components <- readRDS(file.path(tempdir(), "Nat_Gen_component_parameters.rds"))
       }
       para_df <- get_tidy_parameter(cn_components)
-    } else {
+    } else if (startsWith(method, "W")) {
       para_df <- feature_setting
+    } else if (startsWith(method, "T")) {
+      para_df <- "Message: No this info for method T."
     }
 
-    list(
-      features = cn_features,
-      components = cn_components,
-      parameters = para_df,
-      nmf_matrix = cn_matrix
-    )
+    if (startsWith(method, "T")) {
+      res_list <- list(
+        features = cn_features,
+        components = cn_components,
+        parameters = para_df,
+        nmf_matrix = cn_matrix,
+        all_matrices = list(
+          standard_matrix = cn_matrix_list$s_mat,
+          complex_matrix = cn_matrix_list$c_mat
+        )
+      )
+    } else {
+      res_list <- list(
+        features = cn_features,
+        components = cn_components,
+        parameters = para_df,
+        nmf_matrix = cn_matrix
+      )
+    }
+
+    return(res_list)
   }
 }
 
 #' @describeIn sig_tally Returns SBS mutation sample-by-component matrix and APOBEC enrichment
-#' @inheritParams maftools::trinucleotideMatrix
 #' @param mode type of mutation matrix to extract, can be one of 'SBS', 'DBS' and 'ID'.
-#' @param genome_build genome build 'hg19' or 'hg38', if not set, guess it by `ref_genome`.
+#' @param ref_genome 'BSgenome.Hsapiens.UCSC.hg19', 'BSgenome.Hsapiens.UCSC.hg38' and
+#' 'BSgenome.Mmusculus.UCSC.mm10' etc.
+#' @param genome_build genome build 'hg19', 'hg38' or "mm10", if not set, guess it by `ref_genome`.
 #' @param add_trans_bias if `TRUE`, consider transcriptional bias categories.
 #' 'T:' for Transcribed (the variant is on the transcribed strand);
 #' 'U:' for Un-transcribed (the variant is on the untranscribed strand);
@@ -381,7 +370,7 @@ sig_tally.CopyNumber <- function(object,
 #' @references Bergstrom EN, Huang MN, Mahto U, Barnes M, Stratton MR, Rozen SG, Alexandrov LB: SigProfilerMatrixGenerator: a tool for visualizing and exploring patterns of small mutational events. BMC Genomics 2019, 20:685 https://bmcgenomics.biomedcentral.com/articles/10.1186/s12864-019-6041-2
 #' @export
 sig_tally.MAF <- function(object, mode = c("SBS", "DBS", "ID", "ALL"),
-                          ref_genome = NULL,
+                          ref_genome = "BSgenome.Hsapiens.UCSC.hg19",
                           genome_build = NULL,
                           add_trans_bias = FALSE,
                           ignore_chrs = NULL,
@@ -394,19 +383,8 @@ sig_tally.MAF <- function(object, mode = c("SBS", "DBS", "ID", "ALL"),
 
   mode <- match.arg(mode)
 
-  if (is.null(genome_build)) {
-    if (grepl("hg19", ref_genome)) {
-      genome_build <- "hg19"
-    } else if (grepl("hg38", ref_genome)) {
-      genome_build <- "hg38"
-    } else {
-      send_stop("Cannot guess the genome build, please set it by hand!")
-    }
-  }
-
   hsgs.installed <- BSgenome::installed.genomes(splitNameParts = TRUE)
   data.table::setDT(x = hsgs.installed)
-  # hsgs.installed = hsgs.installed[organism %in% "Hsapiens"]
 
   if (nrow(hsgs.installed) == 0) {
     send_stop("Could not find any installed BSgenomes. Use {.code BSgenome::available.genomes()} for options.")
@@ -426,6 +404,18 @@ sig_tally.MAF <- function(object, mode = c("SBS", "DBS", "ID", "ALL"),
     }
   }
 
+  if (is.null(genome_build)) {
+    if (grepl("hg19", ref_genome)) {
+      genome_build <- "hg19"
+    } else if (grepl("hg38", ref_genome)) {
+      genome_build <- "hg38"
+    } else if (grepl("mm10$", ref_genome)) {
+      genome_build <- "mm10"
+    } else {
+      send_stop("Cannot guess the genome build, please set it by hand!")
+    }
+  }
+
   ref_genome <- BSgenome::getBSgenome(genome = ref_genome)
   send_success("Reference genome loaded.")
 
@@ -453,10 +443,10 @@ sig_tally.MAF <- function(object, mode = c("SBS", "DBS", "ID", "ALL"),
     ignore.case = TRUE
   )
   ## Make sure all have prefix
-  if (any(!grepl("chr", query$Chromosome))) {
-    query$Chromosome[!grepl("chr", query$Chromosome)] <-
-      paste0("chr", query$Chromosome[!grepl("chr", query$Chromosome)])
-  }
+  query$Chromosome <- ifelse(startsWith(query$Chromosome, "chr"),
+    query$Chromosome,
+    paste0("chr", query$Chromosome)
+  )
 
   send_success("Chromosome names checked.")
 
@@ -475,13 +465,6 @@ sig_tally.MAF <- function(object, mode = c("SBS", "DBS", "ID", "ALL"),
     ignore.case = TRUE
   )
 
-  query$Chromosome <- sub(
-    pattern = "MT",
-    replacement = "M",
-    x = as.character(query$Chromosome),
-    ignore.case = TRUE
-  )
-
   # detect and transform chromosome 23 to "X"
   query$Chromosome <- sub("23", "X", query$Chromosome)
   # detect and transform chromosome 24 to "Y"
@@ -498,6 +481,20 @@ sig_tally.MAF <- function(object, mode = c("SBS", "DBS", "ID", "ALL"),
   ref_seqs_lvls <- BSgenome::seqnames(x = ref_genome)
   query_seq_lvls_missing <- query_seq_lvls[!Chromosome %in% ref_seqs_lvls]
 
+  if (nrow(query_seq_lvls_missing) > 3) {
+    ## Some reference genome builds have no 'chr' prefix
+    send_warning("Too many chromosome names cannot match reference genome. Try dropping 'chr' prefix to fix it...")
+    query$Chromosome <- sub(
+      pattern = "chr",
+      replacement = "",
+      x = as.character(query$Chromosome),
+      ignore.case = TRUE
+    )
+    query_seq_lvls <- query[, .N, Chromosome]
+    query_seq_lvls_missing <- query_seq_lvls[!Chromosome %in% ref_seqs_lvls]
+    send_info("Dropped.")
+  }
+
   if (nrow(query_seq_lvls_missing) > 0) {
     send_warning(paste0(
       "Chromosome names in MAF must match chromosome names in reference genome.\nIgnorinig ",
